unit UDOADemo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  {$IFDEF ConditionalExpressions}{$if CompilerVersion >= 14} variants,{$IFEND}{$ENDIF} //Delphi 6 or above
  Dialogs, Oracle, StdCtrls, UFlexCelImport, UExcelAdapter, XLSAdapter,
  UCustomFlexCelReport, UFlexCelReport, Grids, UFlexCelGrid, ExtCtrls,
  ActnList, ImgList, ComCtrls, ToolWin, XlsBaseTemplateStore, TemplateStore,
  UDOAFlexCelReport, DB, OracleData, UFormatColumns,UFlxFormats, UWaitCursor;

type
  TFDOADemo = class(TForm)
    MainSession: TOracleSession;
    MainLogon: TOracleLogon;
    CustomSQL: TOracleQuery;
    Splitter1: TSplitter;
    FlexCelGrid: TFlexCelGrid;
    DOAReport: TDOAFlexCelReport;
    XLSAdapter: TXLSAdapter;
    FlexCelImport: TFlexCelImport;
    ToolBar: TToolBar;
    btnGo: TToolButton;
    btnClose: TToolButton;
    ImageList1: TImageList;
    ImageList2: TImageList;
    ImageList3: TImageList;
    ActionList: TActionList;
    ActionExport: TAction;
    ActionExit: TAction;
    StatusBar: TStatusBar;
    XlsSaveDialog: TSaveDialog;
    XlsTemplateStore: TXlsTemplateStore;
    ActionLogon: TAction;
    btnLogon: TToolButton;
    Panel3: TPanel;
    Panel6: TPanel;
    Panel4: TPanel;
    Panel2: TPanel;
    edDateFormat: TEdit;
    Panel5: TPanel;
    Panel1: TPanel;
    MemoSQL: TMemo;
    Splitter2: TSplitter;
    ActionExportDataSet: TAction;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    CustomDataSet: TOracleDataSet;
    DataSetReport: TFlexCelReport;
    XLSAdapterNoTStore: TXLSAdapter;
    chColumnFormat: TCheckBox;
    procedure ActionExitExecute(Sender: TObject);
    procedure ActionExportExecute(Sender: TObject);
    procedure ActionLogonExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ActionExportDataSetExecute(Sender: TObject);
    procedure DOAReportGetCellValueX(Sender: TObject;
      const ExcelApp: TExcelFile; const FieldName: WideString;
      var FieldValue: Variant; var XF: Integer);
  private
    FormatColumns: TFormatColumns;
    procedure Report(const Flx: TCustomFlexCelReport);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FDOADemo: TFDOADemo;

implementation

{$R *.dfm}

procedure TFDOADemo.ActionExitExecute(Sender: TObject);
begin
  Close;
end;

function SkipCR(const s: string): string;
var
  i:integer;
begin
  SetLength(Result, Length(s));
  for i:=1 to length(s) do if s[i]<#32 then Result[i]:=' ' else Result[i]:=s[i];
end;

procedure TFDOADemo.Report(const Flx: TCustomFlexCelReport);
var
  StartTime, EndTime: int64;
  WaitCursor: IWaitCursor;
begin
  if not XlsSaveDialog.Execute then exit;

  StatusBar.SimpleText:='';

  if FileExists(XlsSaveDialog.FileName) then DeleteFile(XlsSaveDialog.FileName);
  Flx.FileName:=XlsSaveDialog.FileName;
  Flx.Values['CurrentDate']:= double(Now); //Dates should be passed as numbers, and the corresponding cell in excel have Date format.
  Flx.Values['CurrentSQL']:= SkipCR(MemoSQL.Lines.Text);
  Flx.Values['DsName']:= MainSession.LogonDatabase ;
  Flx.DateTimeFormat:=edDateFormat.Text;

  if chColumnFormat.Checked then
  begin
    if FormatColumns=nil then FormatColumns:=TFormatColumns.Create(Self);
    CustomSQL.SQL:=MemoSQL.Lines;
    FormatColumns.LoadData(CustomSQL);
    if FormatColumns.ShowModal<>mrOk then exit;
    Flx.OnGetCellValueX:=DOAReportGetCellValueX;
  end else
    Flx.OnGetCellValueX:=nil;

  WaitCursor:=TWaitCursor.Create;

  StartTime:=GetTickCount;
    Flx.Run;
  EndTime:=GetTickCount;

  FlexCelImport.OpenFile(XlsSaveDialog.FileName);
  FlexCelGrid.LoadSheet;
  StatusBar.SimpleText:=Format('Last Report took: %f seconds', [(EndTime-StartTime)/1000]);
end;

procedure TFDOADemo.ActionExportExecute(Sender: TObject);
begin
  if not MainSession.Connected then ActionLogon.Execute;
  if not MainSession.Connected then exit;
  CustomSQL.SQL:=MemoSQL.Lines;

  Report(DOAReport);
end;

procedure TFDOADemo.ActionLogonExecute(Sender: TObject);
begin
  // Log on or off
  if MainSession.Connected then MainSession.LogOff else MainLogon.Execute;
  // The button shows if we're connected
  btnLogon.Down := MainSession.Connected;
end;

procedure TFDOADemo.FormCreate(Sender: TObject);
begin
  edDateFormat.Text:=DOAReport.DateTimeFormat;
end;

procedure TFDOADemo.ActionExportDataSetExecute(Sender: TObject);
begin
  if not MainSession.Connected then ActionLogon.Execute;
  if not MainSession.Connected then exit;
  CustomDataSet.SQL:=MemoSQL.Lines;

  Report(DataSetReport);
end;



procedure TFDOADemo.DOAReportGetCellValueX(Sender: TObject;
  const ExcelApp: TExcelFile; const FieldName: WideString;
  var FieldValue: Variant; var XF: Integer);
var
  s:string;
  Fmt:TFlxFormat;
begin
  // if the stringgrid on uformatcolumns has a format for this cell, use it
  s:=FormatColumns.StringGridFormat.Cells[1, CustomSQL.FieldIndex(FieldName)];
  if s<>'' then
  begin
    Fmt:=ExcelApp.FormatList[XF];
    Fmt.Format:=s;
    XF:= ExcelApp.AddFormat(Fmt);
  end;
end;

end.
