{
   Firebird Library
   Open Source Library No Data Aware for direct access to Firebird
   Relational Database from Borland Delphi / Kylix and Freepascal
   
   File:FBLService.pas
   Copyright (c) 2004 Alessandro Batisti
   fblib@altervista.org
   http://fblib.altervista.org

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
}

{$I fbl.inc}
unit FBLService;

interface

uses
  Classes, SysUtils, ibase_h;

type
  TServiceProtocolType = (ptLocal, ptTcpIp, ptNetBeui);
  TBackupOption = (bkpVerbose,bkpIgnoreCheckSum,bkpIgnoreLimbo,bkpMetadataOnly,
                   bkpNoGarbageCollect,bkpOldDescription,bkpNoTrasportable,bkpConvert);
  TBackupOptions = set of TBackupOption;

  TRestoreOption = (resVerbose,resDeactivateIdx,resNoShadow,resNoValidity,
                    resOneAtATime,resReplace,resCreate,resUseAllSpace,
                    resAccessModeReadOnly,resAccessModeReadWrite);
  TRestoreOptions = set of TRestoreOption;

  TStatOption = (stsDataPages,stsDbLog,stsHdrPages,stsIdxPages,stsSysRelations);
  TStatOptions = set of TStatOption;

  TOnWriteOutput = procedure(Sender: TObject; TextLine: string; IscAction: Integer) of object; //see ibase_h.pas for value of isc_action_*


  TFBLService = class(TComponent)
  private
    FLibraryLoaded: Boolean;
    FUser: string;
    FPassword: string;
    FProtocol: TServiceProtocolType;
    FHost: string;
    FTcpPort: Word;
    FServiceHandle: TISC_SVC_HANDLE;
    FOnWriteOutput :TOnWriteOutput;
    FUserList: TStringList;
    function GetConnected: Boolean;
    function GetUserNames: TStringList;
    procedure AddModifyUser(AIscAction: Char; const AUserName,APassword: string; AFirstName: string = ''; AMiddleName: string = '';
      ALastName: string = ''; AUserID: LongInt = 0; AGroupID: LongInt = 0);
    procedure WriteLineOutput(const AIscAction: Integer);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Connect;
    procedure Disconnect;
    procedure Backup(const ADatabaseFile,ABackupFile: string;
      AOption: TBackupOptions = [] );
    procedure Restore(const ABackupFile,ADatabaseFile: string;
      AOption: TRestoreOptions = []; APageSize: Integer = 0);
    procedure GetLogFile;
    procedure GetStatusReports(const ADatabaseFile: string; AOption: TStatOptions = []);
    procedure AddUser(const AUserName,APassword: string; AFirstName: string = ''; AMiddleName: string = '';
      ALastName: string = ''; AUserID: LongInt = 0; AGroupID: LongInt = 0);
    procedure ModifyUser(const AUserName,APassword: string; AFirstName: string = ''; AMiddleName: string = '';
      ALastName: string = ''; AUserID: LongInt = 0; AGroupID: LongInt = 0);
    procedure DeleteUser(const AUserName: string);
    procedure ViewUser(const AUserName: string ;var AFirstName,AMiddleName,ALastName: string;
      var AUserID,AGroupId: LongInt);
    property Connected: Boolean read GetConnected;
    property UserNames: TStringList read GetUserNames;
  published
    property User: string read FUser write FUser;
    property Password: string read FPassword write FPassword;
    property Protocol: TServiceProtocolType read FProtocol write FProtocol;
    property Host: string read FHost write FHost;
    property TcpPort: Word read FTcpPort write FTcpPort default 3050;
    property OnWriteOutput: TOnWriteOutput read FOnWriteOutput write FOnWriteOutput;
  end;


implementation

uses FBLExcept, FBLmixf ,FBLConst;

//------------------------------------------------------------------------------

constructor TFBLService.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLibraryLoaded := False;
  CheckIbSqlLoaded;
  FLibraryLoaded := True;
  CheckIbSqlLoaded;
  FTcpPort := 3050;
  FServiceHandle := nil;
  FUserList := nil;
end;

//------------------------------------------------------------------------------

destructor TFBLService.Destroy;
begin
  if FLibraryLoaded then
  begin
    if Connected then Disconnect;
    if Assigned(FUserList) then
      FUserList.Free;
  end;
  inherited Destroy;
end;

//------------------------------------------------------------------------------

function TFBLService.GetConnected: Boolean;
begin
  Result := (FServiceHandle <> nil);
end;

//------------------------------------------------------------------------------

procedure TFBLService.WriteLineOutput(const AIscAction: Integer);
var
  RequestInfo: Char;
  BufferResult: array[0..511] of Char;
  Status_vector: ISC_STATUS_VECTOR;
  LenLine: Integer;
  Line: string;
begin
  RequestInfo := Char(isc_info_svc_line);
  FillChar(BufferResult, SizeOf(BufferResult), #0);
  LenLine := 0;
  repeat
    if isc_service_query(@Status_vector,@FServiceHandle,nil,0,nil,
      1,@RequestInfo,SizeOf(BufferResult),
      BufferResult) <> 0 then
         FBLShowError(@Status_vector);
    if BufferResult[0] = Char(isc_info_svc_line) then
    begin
      LenLine := isc_vax_integer(@BufferResult[1],2);
      SetLength(Line,LenLine);
      Move(BufferResult[3],Line[1],LenLine);
      If Assigned(FOnWriteOutput) then
        FOnWriteOutput(self,Line,AIscAction);
    end;
  until LenLine = 0
end;

//------------------------------------------------------------------------------

procedure TFBLService.Connect;
var
  SpbBuffer: PChar;
  SpbIdx: Short;
  Status_vector: ISC_STATUS_VECTOR;
  ServiceName: string;
  LenBuffer: Integer;
begin
  if FServiceHandle <> nil then
    FBLError(E_SM_ALREADY_CON);
  case FProtocol of
    ptTcpIp:
     begin
       if FTcpPort <> 3050 then
         ServiceName := Format('%s/%d:service_mgr',[FHost,FTcpPort])
       else
         ServiceName := Format('%s:service_mgr',[FHost]);
     end;
    ptNetBeui: ServiceName := '\\' + FHost + '\' + 'service_mgr';
    else  ServiceName :=  'service_mgr';
  end;
  SpbBuffer := nil;
  LenBuffer := 4 + Length(FUser) + 2 +  Length(FPassword);
  FBLMalloc(SpbBuffer,LenBuffer);
  SpbIdx := 0;
  try
    SpbBuffer[SpbIdx] := Char(isc_spb_version);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(isc_spb_current_version);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(isc_spb_user_name);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Length(FUser));
    Inc(SpbIdx);
    Move(FUser[1],SpbBuffer[SpbIdx],Length(FUser));
    Inc(SpbIdx, Length(FUser));
    SpbBuffer[SpbIdx] := Char(isc_spb_password);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Length(FPassword));
    Inc(SpbIdx);
    Move(FPassword[1],SpbBuffer[SpbIdx],Length(FPassword));
    Inc(SpbIdx,Length(FPassword));
    if isc_service_attach(@Status_vector,0,PChar(ServiceName),@FServiceHandle,
      SpbIdx,SpbBuffer) <>  0 then
        FBLShowError(@Status_vector);
  finally
    FBLFree(SpbBuffer);
  end;  
end;

//------------------------------------------------------------------------------

procedure TFBLService.Disconnect;
var
  Status_vector: ISC_STATUS_VECTOR;
begin
  if FServiceHandle = nil then
    FBLError(E_SM_NO_CON);
  if isc_service_detach(@Status_vector,@FServiceHandle) <> 0 then
    FBLShowError(@Status_vector);
end;

//------------------------------------------------------------------------------

procedure TFBLService.Backup(const ADatabaseFile,ABackupFile: string;
  AOption: TBackupOptions = [] );
var
  SpbBuffer: PChar;
  SpbIdx: Short;
  LenBuffer: Integer;
  Status_vector: ISC_STATUS_VECTOR;
  Options : Cardinal;   //bitmask of isc_spb_options
begin
  if FServiceHandle = nil then
    FBLError(E_SM_NO_CON);
  Options := 0;
  SpbBuffer := nil;
  SpbIdx := 0;
  { LenBuffer
    1 byte : isc_action_svc_backup
    1 byte : isc_spb_dbname
    2 byte : lenght DatabaseFile
    n byte : Databasefile name
    1 byte : isc_spb_bkp_file
    2 byte : lenght backupfile
    n byte : backupfile name
    1 byte : isc_spb_options
    4 byte : Options bitmask
    + 1 byte if verbose request}
  LenBuffer := 4 + Length(ADatabaseFile) + 3 + Length(ABackupFile) + 5;
  if  bkpVerbose in AOption then
    Inc(LenBuffer);
  FBLMalloc(SpbBuffer,LenBuffer);
  try
    SpbBuffer[SpbIdx] :=  Char(isc_action_svc_backup);
    Inc(SpbIdx);
    //databasefile
    SpbBuffer[SpbIdx] := Char(isc_spb_dbname);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Length(ADatabaseFile));
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Length(ADatabaseFile) shr 8);
    Inc(SpbIdx);
    Move(ADatabaseFile[1],SpbBuffer[SpbIdx],Length(ADatabaseFile));
    Inc(SpbIdx,Length(ADatabaseFile));
    // verbose result
    if  bkpVerbose in AOption then
    begin
      SpbBuffer[SpbIdx] := Char(isc_spb_verbose);
      Inc(SpbIdx);
    end;
    //backupfile
    SpbBuffer[SpbIdx] := Char(isc_spb_bkp_file);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Length(ABackupFile));
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Length(ABackupFile) shr 8);
    Inc(SpbIdx);
    Move(ABackupFile[1],SpbBuffer[SpbIdx],Length(ABackupFile));
    Inc(SpbIdx,Length(ABackupFile));
    //options backup
    if bkpIgnoreCheckSum in  AOption then
      Options := Options or isc_spb_bkp_ignore_checksums;
    if bkpIgnoreLimbo in AOption then
      Options := Options or isc_spb_bkp_ignore_limbo;
    if bkpMetadataOnly in AOption then
      Options := Options or isc_spb_bkp_metadata_only;
    if bkpNoGarbageCollect in AOption then
      Options := Options or isc_spb_bkp_no_garbage_collect;
    if bkpOldDescription in AOption then
      Options := Options or isc_spb_bkp_old_descriptions;
    if bkpConvert in AOption then
      Options := Options or isc_spb_bkp_convert;
    SpbBuffer[SpbIdx] := Char(isc_spb_options);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Options);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Options shr 8);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Options shr 16);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Options shr 24);
    Inc(SpbIdx);
   // start backup
    if isc_service_start(@Status_vector,@FServiceHandle,nil,SpbIdx,
      SpbBuffer) <> 0 then
        FBLShowError(@Status_vector);
  finally
    FBLFree(SpbBuffer);
  end;
  // if Verbose print msg
  if bkpVerbose in AOption then
    WriteLineOutput(isc_action_svc_backup);
end;

//------------------------------------------------------------------------------

procedure TFBLService.Restore(const ABackupFile,ADatabaseFile: string;
      AOption: TRestoreOptions = []; APageSize: Integer = 0);
var
  SpbBuffer: PChar;
  SpbIdx: Short;
  LenBuffer: Integer;
  Status_vector: ISC_STATUS_VECTOR;
  Options : Cardinal;
begin
  if FServiceHandle = nil then
    FBLError(E_SM_NO_CON);
  if  not((resCreate in AOption) or (resReplace in AOption)) then
    FBLError(E_SM_RES_NO_ACTION);
  SpbBuffer := nil;
  Options := 0;
  SpbIdx := 0;
  { LenBuffer
    1 byte : isc_action_svc_restore
    1 byte : isc_spb_bkp_file
    2 byte : lenght BackupFile
    n byte : Backup name
    1 byte : isc_spb_bkp_dbname
    2 byte : lenght DatabaseFile
    n byte : DatabaseFile name
    1 byte : isc_spb_options
    4 byte : Options bitmask
    + 1 byte if verbose request
    + 1 byte if access mode
    + 5 byte if Pagesize > 0}
  LenBuffer := 4 + Length(ABackupFile) + 3 + Length(ADatabaseFile) + 5;
  if (resVerbose in Aoption) then
    Inc(LenBuffer);
  if (resAccessModeReadOnly in AOption) or (resAccessModeReadWrite in AOption) then
    Inc(LenBuffer);
  if APageSize > 0 then
    Inc(LenBuffer,5);
  FBLMalloc(spbBuffer,LenBuffer);
  try
    SpbBuffer[SpbIdx] :=  Char(isc_action_svc_restore);
    Inc(SpbIdx);
    //backupfile
    SpbBuffer[SpbIdx] := Char(isc_spb_bkp_file);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Length(ABackupFile));
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Length(ABackupFile) shr 8);
    Inc(SpbIdx);
    Move(ABackupFile[1],SpbBuffer[SpbIdx],Length(ABackupFile));
    Inc(SpbIdx,Length(ABackupFile));
    // verbose result
    if resVerbose in AOption then
    begin
      SpbBuffer[SpbIdx] := Char(isc_spb_verbose);
      Inc(SpbIdx);
    end;
    //databasefile
    SpbBuffer[SpbIdx] := Char(isc_spb_dbname);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Length(ADatabaseFile));
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Length(ADatabaseFile) shr 8);
    Inc(SpbIdx);
    Move(ADatabaseFile[1],SpbBuffer[SpbIdx],Length(ADatabaseFile));
    Inc(SpbIdx,Length(ADatabaseFile));
    //options
    if resDeactivateIdx in AOption then
      Options := Options or isc_spb_res_deactivate_idx;
    if resNoShadow in AOption then
      Options := Options or isc_spb_res_no_shadow;
    if resNoValidity in AOption then
      Options := Options or isc_spb_res_no_validity;
    if resOneAtATime in AOption then
      Options := Options or isc_spb_res_one_at_a_time;
    if resReplace in AOption then
      Options := Options or isc_spb_res_replace;
    if resCreate in AOption then
      Options := Options or isc_spb_res_create;
    if resUseAllSpace in AOption then
      Options := Options or isc_spb_res_use_all_space;
    SpbBuffer[SpbIdx] := Char(isc_spb_options);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Options);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Options shr 8);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Options shr 16);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Options shr 24);
    Inc(SpbIdx);
    if resAccessModeReadOnly in  AOption then
    begin
      if resAccessModeReadWrite in AOption then
        FBLError(E_SM_RES_PARAMS_ACCESSMODE);
      SpbBuffer[SpbIdx] := Char(isc_spb_res_access_mode);
      Inc(SpbIdx);
      SpbBuffer[SpbIdx] := Char(isc_spb_res_am_readonly);
    end;
    if resAccessModeReadWrite in AOption then
    begin
      if resAccessModeReadOnly in AOption then
        FBLError(E_SM_RES_PARAMS_ACCESSMODE);
      SpbBuffer[SpbIdx] := Char(isc_spb_res_access_mode);
      Inc(SpbIdx);
      SpbBuffer[SpbIdx] := Char(isc_spb_res_am_readwrite);
    end;
    // Page Size
    if APageSize <> 0 then
    begin
     SpbBuffer[SpbIdx] := Char(isc_spb_res_page_size);
     Inc(SpbIdx);
     SpbBuffer[SpbIdx] := Char(APageSize);
     Inc(SpbIdx);
     SpbBuffer[SpbIdx] := Char(APageSize shr 8);
     Inc(SpbIdx);
     SpbBuffer[SpbIdx] := Char(APageSize shr 16);
     Inc(SpbIdx);
     SpbBuffer[SpbIdx] := Char(APageSize shr 24);
     Inc(SpbIdx);
    end;
    // start restore
    if isc_service_start(@Status_vector,@FServiceHandle,nil,SpbIdx,
      SpbBuffer) <> 0 then
        FBLShowError(@Status_vector);
  finally
    FBLFree(SpbBuffer);
  end;
  if resVerbose in AOption then
    WriteLineOutput(isc_action_svc_restore);
end;

//------------------------------------------------------------------------------

procedure TFBLService.GetLogFile;
var
  RequestInfo: Char;
  Status_vector: ISC_STATUS_VECTOR;
begin
  if FServiceHandle = nil then
    FBLError(E_SM_NO_CON);
  RequestInfo := Char(isc_action_svc_get_ib_log);
  if isc_service_start(@Status_vector,@FServiceHandle,nil,1,
    @RequestInfo) <> 0 then
      FBLShowError(@Status_vector);
  WriteLineOutput(isc_action_svc_get_ib_log);
end;

//------------------------------------------------------------------------------

procedure TFBLService.GetStatusReports(const ADatabaseFile: string; AOption: TStatOptions = []);
var
  SpbBuffer: PChar;
  SpbIdx: Short;
  LenBuffer: Integer;
  Status_vector: ISC_STATUS_VECTOR;
  Options: Cardinal;
begin
  if FServiceHandle = nil then
    FBLError(E_SM_NO_CON);
  SpbBuffer := nil;
  Options := 0;
  SpbIdx := 0;
  { LenBuffer
    1 byte: isc_action_svc_db_stats
    2 byte: Length DatabaseFile
    n byte: DatabaseFile Name
    1 byte: isc_spb_options
    4 byte: BitMask Options }
  LenBuffer := 4 + Length(ADatabaseFile) + 5;
  FBLMalloc(SpbBuffer,LenBuffer);
  try
    SpbBuffer[SpbIdx] := Char(isc_action_svc_db_stats);
    Inc(SpbIdx);
    //databasefile
    SpbBuffer[SpbIdx] := Char(isc_spb_dbname);
    Inc(SpbIdx);
    SpbBuffer[SpbIdx] := Char(Length(ADatabaseFile));
    Inc(spbidx);
    SpbBuffer[SpbIdx] := Char(Length(ADatabaseFile) shr 8);
    Inc(SpbIdx);
    Move(ADatabaseFile[1],SpbBuffer[spbidx],Length(ADatabaseFile));
    Inc(SpbIdx,Length(ADatabaseFile));
    //options (stsDataPages,stsDbLog,stsHdrPages,stsIdxPages,stsSysRelations)
    if stsDataPages in AOption then
      Options := Options or isc_spb_sts_data_pages;
    if stsDbLog in AOption then
      Options := Options or isc_spb_sts_db_log;
    if stsHdrPages in AOption then
      Options := Options or isc_spb_sts_hdr_pages;
    if stsIdxPages in AOption then
      Options := Options or isc_spb_sts_idx_pages;
    if stsSysRelations in AOption then
      Options := Options or isc_spb_sts_sys_relations;
    SpbBuffer[SpbIdx] := Char(isc_spb_options);
    Inc(spbidx);
    SpbBuffer[SpbIdx] := Char(Options);
    Inc(spbidx);
    SpbBuffer[SpbIdx] := Char(Options shr 8);
    Inc(spbidx);
    SpbBuffer[SpbIdx] := Char(Options shr 16);
    Inc(spbidx);
    SpbBuffer[SpbIdx] := Char(Options shr 24);
    Inc(SpbIdx);
    // start service
    if isc_service_start(@Status_vector,@FServiceHandle,nil,SpbIdx,
      SpbBuffer) <> 0 then
        FBLShowError(@Status_vector);
  finally
    FBLFree(SpbBuffer);
  end;
  WriteLineOutput(isc_action_svc_db_stats);
end;

//------------------------------------------------------------------------------

function TFBLService.GetUserNames: TStringList;
var
  RequestInfo: Char;
  BufferResult: array[0..255] of Char; //tempBuffer
  Buffer: PChar;                       //Full Buffer
  Status_vector: ISC_STATUS_VECTOR;
  User: string;
  Idx,TotBuffer,PosBuffer,LenResult: Integer;
begin
  if FServiceHandle = nil then
    FBLError(E_SM_NO_CON);
  if FUserList = nil then FUserList := TStringList.Create;
  Result := FUserList;
  FUserList.Clear;
  RequestInfo := Char(isc_action_svc_display_user);
  if isc_service_start(@Status_vector,@FServiceHandle,nil,1,
      @RequestInfo) <> 0 then
        FBLShowError(@Status_vector);
  RequestInfo := Char(isc_info_svc_get_users);
  FillChar(BufferResult,SizeOf(BufferResult),#0);
  Buffer := nil;
  TotBuffer := 0;
  PosBuffer := 0;
  try
    repeat
      if isc_service_query(@Status_vector,@FServiceHandle,nil,0,nil,
        1,@RequestInfo,SizeOf(BufferResult),
          BufferResult) <> 0 then
            FBLShowError(@Status_vector);
      LenResult := isc_vax_integer(@BufferResult[1],2);
      if LenResult = 0 then
      begin
        Inc(TotBuffer,1);
        ReallocMem(Buffer,TotBuffer);
        Buffer[TotBuffer -1] := Char(isc_info_end);
      end
      else
      begin
        Inc(TotBuffer,LenResult);
        ReallocMem(Buffer,TotBuffer);
        Move(BufferResult[3],Buffer[PosBuffer],LenResult);
        Inc(PosBuffer,LenResult);
      end;
    until LenResult = 0;
    idx := 0;
    while (Buffer[idx] <> Char(isc_info_end)) do
    begin
     case  Integer(Buffer[idx]) of
       isc_spb_sec_userid:
        Inc(idx,5);
       isc_spb_sec_groupid:
        Inc(idx,5);
       isc_spb_sec_username:
       begin
         Inc(idx);
         LenResult:=isc_vax_integer(@Buffer[Idx],2);
         Inc(idx,2);
         SetLength(User,LenResult);
         Move(Buffer[idx],User[1],LenResult);
         FUserList.Add(User);
         Inc(idx,LenResult);
       end;
       isc_spb_sec_groupname:
        begin
         Inc(idx);
         LenResult:=isc_vax_integer(@Buffer[Idx],2);
         Inc(idx,2);
         Inc(idx,LenResult);
       end;
       isc_spb_sec_firstname:
       begin
         Inc(idx);
         LenResult:=isc_vax_integer(@Buffer[Idx],2);
         Inc(idx,2);
         Inc(idx,LenResult);
       end;
       isc_spb_sec_middlename:
       begin
         Inc(idx);
         LenResult := isc_vax_integer(@Buffer[Idx],2);
         Inc(idx,2);
         Inc(idx,LenResult);
       end;
       isc_spb_sec_lastname:
       begin
         Inc(idx);
         LenResult:=isc_vax_integer(@Buffer[Idx],2);
         Inc(idx,2);
         Inc(idx,LenResult);
        end;
      end;
    end;
  finally
    ReallocMem(Buffer,0);
  end;
end;


//------------------------------------------------------------------------------
{

function TFBLService.GetUserNames: TStringList;
var
  RequestInfo: Char ;
  BufferResult: array[0..16383] of Char;
  Status_vector: ISC_STATUS_VECTOR;
  LenResult :Integer;
  Idx: Integer;
  LenData: Integer;
  User: string;
begin
  if FServiceHandle = nil then
    FBLError(E_SM_NO_CON);
  if FUserList = nil then FUserList := TStringList.Create;
  Result := FUserList;
  FUserList.Clear;
  RequestInfo := Char(isc_action_svc_display_user);
  Idx := 0;
  FillChar(BufferResult,SizeOf(BufferResult),#0);
  if isc_service_start(@Status_vector,@FServiceHandle,nil,1,
      @RequestInfo) <> 0 then
        FBLShowError(@Status_vector);
  RequestInfo := Char(isc_info_svc_get_users);
  if isc_service_query(@Status_vector,@FServiceHandle,nil,0,nil,
      1,@RequestInfo,SizeOf(BufferResult),
      BufferResult) <> 0 then
        FBLShowError(@Status_vector);
   Inc(idx);
   LenResult:=isc_vax_integer(@BufferResult[Idx],2);
   If LenResult > Sizeof(BufferResult) then FBLError('error');
   Inc(Idx,2);
   while (BufferResult[idx] <> Char(isc_info_end)) do
   begin
     case  Integer(BufferResult[idx]) of
       isc_spb_sec_userid:
        Inc(idx,5);
       isc_spb_sec_groupid:
        Inc(idx,5);
       isc_spb_sec_username:
       begin;
         Inc(idx);
         LenData:=isc_vax_integer(@BufferResult[Idx],2);
         Inc(idx,2);
         SetLength(User,LenData);
         Move(BufferResult[idx],User[1],LenData);
         FuserList.Add(User);
         Inc(idx,LenData);
       end;
       isc_spb_sec_groupname:
        begin;
         Inc(idx);
         LenData:=isc_vax_integer(@BufferResult[Idx],2);
         Inc(idx,2);
         Inc(idx,LenData);
       end;
       isc_spb_sec_firstname:
       begin;
         Inc(idx);
         LenData:=isc_vax_integer(@BufferResult[Idx],2);
         Inc(idx,2);
         Inc(idx,LenData);
       end;
       isc_spb_sec_middlename:
       begin;
         Inc(idx);
         LenData:=isc_vax_integer(@BufferResult[Idx],2);
         Inc(idx,2);
         Inc(idx,LenData);
       end;
       isc_spb_sec_lastname:
       begin;
         Inc(idx);
         LenData:=isc_vax_integer(@BufferResult[Idx],2);
         Inc(idx,2);
         Inc(idx,LenData);
       end;
     end;
   end;
end;

}

//------------------------------------------------------------------------------

procedure TFBLService.AddModifyUser(AIscAction: Char; const AUserName,APassword :string; AFirstName: string = ''; AMiddleName: string = '';
      ALastName: string = ''; AUserID: LongInt = 0; AGroupID: LongInt = 0);
var
  SpbBuffer: array [0..254] of Char ;
  Status_vector: ISC_STATUS_VECTOR;
  SpbBufferIdx: Short;
  UserName,Password,FirstName,LastName,MiddleName: string;
begin
  if FServiceHandle = nil then
    FBLError(E_SM_NO_CON);
  if Length(AUserName) > 31 then
    UserName :=  Copy(AUserName,0,31)
  else
    UserName := AUserName;
  if Length(APassword) > 8 then
    Password :=  Copy(APassword,0,8)
  else
    Password := APassword;
  if Length(AFirstName) > 17 then
    FirstName := Copy(AFirstName,0,17)
  else
    FirstName := AFirstName;
  if Length(AMiddleName) > 17 then
    MiddleName := Copy(AMiddleName,0,17)
  else
    MiddleName := AMiddleName;
  if Length(ALastName) > 17 then
    LastName := Copy(ALastName,0,17)
  else
    LastName := ALastName;
  FillChar(SpbBuffer,SizeOf(SpbBuffer),#0);
  SpbBufferIdx := 0;
  SpbBuffer[SpbBufferIdx] := AIscAction;
  Inc(SpbBufferIdx);
  //UserName
  SpbBuffer[SpbBufferIdx] := Char(isc_spb_sec_username);
  Inc(SpbBufferIdx);
  SpbBuffer[SpbBufferIdx] := Char(Length(UserName));
  Inc(SpbBufferIdx);
  SpbBuffer[SpbBufferIdx] := Char(Length(UserName) shr 8);
  Inc(SpbBufferIdx);
  Move(UserName[1],SpbBuffer[SpbBufferIdx],Length(UserName));
  Inc(SpbBufferIdx,Length(UserName));
  //Password
  SpbBuffer[SpbBufferIdx] := Char(isc_spb_sec_password);
  Inc(SpbBufferIdx);
  SpbBuffer[SpbBufferIdx] := Char(Length(Password));
  Inc(SpbBufferIdx);
  SpbBuffer[SpbBufferIdx] := Char(Length(Password) shr 8);
  Inc(SpbBufferIdx);
  Move(Password[1],SpbBuffer[SpbBufferIdx],Length(Password));
  Inc(SpbBufferIdx,Length(Password));
  //FirstName
  if FirstName <> '' then
  begin
    SpbBuffer[SpbBufferIdx] := Char(isc_spb_sec_firstname);
    Inc(SpbBufferIdx);
    SpbBuffer[SpbBufferIdx] := Char(Length(FirstName));
    Inc(SpbBufferIdx);
    SpbBuffer[SpbBufferIdx] := Char(Length(FirstName) shr 8);
    Inc(SpbBufferIdx);
    Move(FirstName[1],SpbBuffer[SpbBufferIdx],Length(FirstName));
    Inc(SpbBufferIdx,Length(FirstName));
  end;
  //MiddleName
  if MiddleName <> '' then
  begin
    SpbBuffer[SpbBufferIdx] := Char(isc_spb_sec_middlename);
    Inc(SpbBufferIdx);
    SpbBuffer[SpbBufferIdx] := Char(Length(MiddleName));
    Inc(SpbBufferIdx);
    SpbBuffer[SpbBufferIdx] := Char(Length(MiddleName) shr 8);
    Inc(SpbBufferIdx);
    Move(MiddleName[1],SpbBuffer[SpbBufferIdx],Length(MiddleName));
    Inc(SpbBufferIdx,Length(MiddleName));
  end;
  //LastName
  if LastName <> '' then
  begin
    SpbBuffer[SpbBufferIdx] := Char(isc_spb_sec_lastname);
    Inc(SpbBufferIdx);
    SpbBuffer[SpbBufferIdx] := Char(Length(LastName));
    Inc(SpbBufferIdx);
    SpbBuffer[SpbBufferIdx] := Char(Length(LastName) shr 8);
    Inc(SpbBufferIdx);
    Move(LastName[1],SpbBuffer[SpbBufferIdx],Length(LastName));
    Inc(SpbBufferIdx,Length(LastName));
  end;
  //UserId
  if AUserId > 0 then
  begin
    SpbBuffer[SpbBufferIdx] := Char(isc_spb_sec_userid);
    Inc(SpbBufferIdx);
    SpbBuffer[SpbBufferIdx] := Char(AUserId);
    Inc(SpbBufferIdx);
    SpbBuffer[SpbBufferIdx] := Char(AUserId shr 8);
    Inc(SpbBufferIdx);
    SpbBuffer[SpbBufferIdx] := Char(AUserId shr 16);
    Inc(SpbBufferIdx);
    SpbBuffer[SpbBufferIdx] := Char(AUserId shr 24);
    Inc(SpbBufferIdx);
  end;
  //GroupId
  if AGroupID > 0 then
  begin
    SpbBuffer[SpbBufferIdx] := Char(isc_spb_sec_groupid);
    Inc(SpbBufferIdx);
    SpbBuffer[SpbBufferIdx] := Char(AGroupID);
    Inc(SpbBufferIdx);
    SpbBuffer[SpbBufferIdx] := Char(AGroupID shr 8);
    Inc(SpbBufferIdx);
    SpbBuffer[SpbBufferIdx] := Char(AGroupID shr 16);
    Inc(SpbBufferIdx);
    SpbBuffer[SpbBufferIdx] := Char(AGroupID shr 24);
    Inc(SpbBufferIdx);
  end;
  if isc_service_start(@Status_vector,@FServiceHandle,nil,SpbBufferIdx,
      SpbBuffer) <> 0 then
        FBLShowError(@Status_vector);
end;

//------------------------------------------------------------------------------

procedure TFBLService.AddUser(const AUserName,APassword :string; AFirstName: string = ''; AMiddleName: string = '';
      ALastName: string = ''; AUserID: LongInt = 0; AGroupID: LongInt = 0);
begin
  AddModifyUser(Char(isc_action_svc_add_user),AUserName,APassword,AFirstName,AMiddleName,ALastName,AUserID,AGroupID);
end;

//------------------------------------------------------------------------------

procedure TFBLService.ModifyUser(const AUserName,APassword :string; AFirstName: string = ''; AMiddleName: string = '';
      ALastName: string = ''; AUserID: LongInt = 0; AGroupID: LongInt = 0);
begin
  AddModifyUser(Char(isc_action_svc_modify_user),AUserName,APassword,AFirstName,AMiddleName,ALastName,AUserID,AGroupID);
end;

//------------------------------------------------------------------------------

procedure TFBLService.DeleteUser(const AUserName: string);
var
  SpbBuffer: array [0..127] of Char ;
  Status_vector: ISC_STATUS_VECTOR;
  SpbBufferIdx: Short;
  UserName: string;
begin
  if FServiceHandle = nil then
    FBLError(E_SM_NO_CON);
  if Length(AUserName) > 31 then
    UserName :=  Copy(AUserName,0,31)
  else
    UserName := AUserName;
  FillChar(SpbBuffer,SizeOf(SpbBuffer),#0);
  SpbBufferIdx := 0;
  SpbBuffer[SpbBufferIdx] := Char(isc_action_svc_delete_user);
  Inc(SpbBufferIdx);
  //UserName
  SpbBuffer[SpbBufferIdx] := Char(isc_spb_sec_username);
  Inc(SpbBufferIdx);
  SpbBuffer[SpbBufferIdx] := Char(Length(UserName));
  Inc(SpbBufferIdx);
  SpbBuffer[SpbBufferIdx] := Char(Length(UserName) shr 8);
  Inc(SpbBufferIdx);
  Move(UserName[1],SpbBuffer[SpbBufferIdx],Length(UserName));
  Inc(SpbBufferIdx,Length(UserName));
  if isc_service_start(@Status_vector,@FServiceHandle,nil,SpbBufferIdx,
    SpbBuffer) <> 0 then
      FBLShowError(@Status_vector);
end;

//------------------------------------------------------------------------------

procedure TFBLService.ViewUser(const AUserName: string ;var AFirstName,AMiddleName,ALastName: string;
  var AUserID,AGroupId:LongInt);
var
  SpbBuffer: array [0..254] of Char ;
  BufferResult: array[0..254] of Char;
  Status_vector: ISC_STATUS_VECTOR;
  SpbBufferIdx: Short;
  LenData: Integer;
  UserName: string;
begin
  if FServiceHandle = nil then
    FBLError(E_SM_NO_CON);
  if Length(AUserName) > 31 then
    UserName :=  Copy(AUserName,0,31)
  else
    UserName := AUserName;
  FillChar(SpbBuffer,SizeOf(SpbBuffer),#0);
  SpbBufferIdx := 0;
  SpbBuffer[SpbBufferIdx] := Char(isc_action_svc_display_user);
  Inc(SpbBufferIdx);
  SpbBuffer[SpbBufferIdx] := Char(isc_spb_sec_username);
  Inc(SpbBufferIdx);
  SpbBuffer[SpbBufferIdx] := Char(Length(UserName));
  Inc(SpbBufferIdx);
  SpbBuffer[SpbBufferIdx] := Char(Length(UserName) shr 8);
  Inc(SpbBufferIdx);
  Move(UserName[1],SpbBuffer[SpbBufferIdx],Length(UserName));
  Inc(SpbBufferIdx,Length(UserName));
  if isc_service_start(@Status_vector,@FServiceHandle,nil,SpbBufferIdx,
      SpbBuffer) <> 0 then
        FBLShowError(@Status_vector);
  FillChar(BufferResult,SizeOf(BufferResult),#0);
  SpbBuffer[0] := Char(isc_info_svc_get_users);
  if isc_service_query(@Status_vector,@FServiceHandle,nil,0,nil,
      1,SpbBuffer,SizeOf(BufferResult),
      BufferResult) <> 0 then
        FBLShowError(@Status_vector);
  SpbBufferIdx := 3;
  while (BufferResult[SpbBufferIdx] <> Char(isc_info_end)) do
  begin
    case  Integer(BufferResult[SpbBufferIdx]) of
      isc_spb_sec_userid:
        begin
         Inc(SpbBufferIdx);
         AUserID := isc_vax_integer(@BufferResult[SpbBufferIdx],4);
         Inc(SpbBufferIdx,4);
        end;
      isc_spb_sec_groupid:
        begin
         Inc(SpbBufferIdx);
         AGroupID := isc_vax_integer(@BufferResult[SpbBufferIdx],4);
         Inc(SpbBufferIdx,4);
        end;
      isc_spb_sec_username:
       begin;
         Inc(SpbBufferIdx);
         LenData:=isc_vax_integer(@BufferResult[SpbBufferIdx],2);
         Inc(SpbBufferIdx,2 + LenData);
       end;
       isc_spb_sec_groupname:
        begin;
         Inc(SpbBufferIdx);
         LenData:=isc_vax_integer(@BufferResult[SpbBufferIdx],2);
         Inc(SpbBufferIdx,2 + LenData);
       end;
       isc_spb_sec_firstname:
       begin;
         Inc(SpbBufferIdx);
         LenData:=isc_vax_integer(@BufferResult[SpbBufferIdx],2);
         Inc(SpbBufferIdx,2);
         SetLength(AFirstName,LenData);
         Move(BufferResult[SpbBufferIdx],AFirstName[1],LenData);
         Inc(SpbBufferIdx,LenData);
       end;
       isc_spb_sec_middlename:
       begin;
         Inc(SpbBufferIdx);
         LenData:=isc_vax_integer(@BufferResult[SpbBufferIdx],2);
         Inc(SpbBufferIdx,2);
         SetLength(AMiddleName,LenData);
         Move(BufferResult[SpbBufferIdx],AMiddleName[1],LenData);
         Inc(SpbBufferIdx,LenData);
       end;
       isc_spb_sec_lastname:
       begin;
         Inc(SpbBufferIdx);
         LenData:=isc_vax_integer(@BufferResult[SpbBufferIdx],2);
         Inc(SpbBufferIdx,2);
         SetLength(ALastName,LenData);
         Move(BufferResult[SpbBufferIdx],ALastName[1],LenData);
         Inc(SpbBufferIdx,LenData);
       end;
     end;
   end;
end;

end.

